//
// XST005P : R2R VIDEO DAC tester
//
// Copyright(c) 2002 Tatsuyuki Satoh , All rights reserved
//
// Important !
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// This is not GPL.
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source.
//   2)Include all these headers , and add document of 
//     modification points
//
// PSW1 = up   value
// PSW2 = down value
//
// 7SEG = DAC VALUE
//
// Histry
// 2002.10.24 T.Satoh
//

module top(
//XST005 generic assign
	PSW,LED,XSG,SA,BUZZ,PCLK,
//VOUT
	V_OUT,
	TRG
	);

//I/O assign
input [3:0] PSW;
inout [7:0] LED,XSG;
inout [3:0] SA;
output BUZZ;
input PCLK; // 14.138MHz

//VOUT
output [9:0] V_OUT;
output TRG;

//
// 7segment dynamic LED drive
//
wire [3:0] dot;
wire [1:0] led_com;
wire led_en;
wire [3:0] led_cas;
wire [15:0] led_num;

hex4_led seg_led(.HEX4(led_num),.DOT(dot),.CNT(led_com),.EN(led_en),.SEG(XSG),.COM(led_cas) );

assign SA[3] = led_cas[0] ? 1'bz : 1'b0;
assign SA[2] = led_cas[1] ? 1'bz : 1'b0;
assign SA[1] = led_cas[2] ? 1'bz : 1'b0;
assign SA[0] = led_cas[3] ? 1'bz : 1'b0;

// tick counter (PCLK = 18.432MHz)
// ticker[16] = @140Hz
// ticker[24] = @1Hz
reg  [43:0] ticker;

always @(posedge PCLK)
begin
  ticker <= ticker + 1;
end

// dac value
reg [7:0] vout;

//
// composite Video DAC
//
// 9bit fullscale -> 8bit 75ohm/0.75Vpp
// 500/1K/2K/4K/8K/16K/32K/64K
// conversion table
//wire [7:0] vod;
//RAMB4_S8 vtable(.CLK(t4sc),.ADDR(vout),.DO(vod),.DI(8'h00),.EN(1'b1),.WE(1'b0),.RST(1'b0));
//
// output composite video with power drive
//

OBUF_S_24 V0(.O(V_OUT[0]),.I(vout[7]));
OBUF_S_24 V1(.O(V_OUT[1]),.I(vout[6]));
OBUF_S_24 V2(.O(V_OUT[2]),.I(vout[5]));
OBUF_S_24 V3(.O(V_OUT[3]),.I(vout[4]));
OBUF_S_24 V4(.O(V_OUT[4]),.I(vout[3]));
OBUF_S_24 V5(.O(V_OUT[5]),.I(vout[2]));
OBUF_S_24 V6(.O(V_OUT[6]),.I(vout[1]));
OBUF_S_24 V7(.O(V_OUT[7]),.I(vout[0]));
OBUF_S_24 V8(.O(V_OUT[8]),.I(1'b0));
OBUF_S_24 V9(.O(V_OUT[9]),.I(1'b0));

//
// test trigger
//

assign TRG = 1'h1;

//
// user I/F
//
reg [3:0] repeat_cnt;

always @(posedge ticker[18])
begin

	// repeat counter
	if(PSW[3]&PSW[2]&PSW[1]&PSW[0])
		repeat_cnt <= 0;
	else if(repeat_cnt != 4'hf)
		repeat_cnt <= repeat_cnt + 1;
 
	//up / down
	if( (repeat_cnt==0) || (repeat_cnt==4'hf) )
 	begin
		if(~PSW[0])			 
			vout <= vout + 1;
		if(~PSW[1])			 
			vout <= vout - 1;
	end 
end

//
// assign 7segment led : 
//
//
assign led_num = { 8'h00,vout };
assign dot[0] = 1'b0; // sc0
assign dot[1] = 1'b0; // sc1
assign dot[2] = 1'b0; // sc2
assign dot[3] = 1'b0; // sc3
assign led_com = ticker[16:15]; // 140Hz cycle
assign led_en  = !(ticker[14:0] == 0);

//
// static LED x8 driver
//
assign LED[0] = 1'b0;
assign LED[1] = 1'b0;
assign LED[2] = 1'b0;
assign LED[3] = 1'b0;
assign LED[4] = 1'b0;
assign LED[5] = 1'b0;
assign LED[6] = 1'b0;
assign LED[7] = 1'b0;

//
// BUZZER driver
//
assign BUZZ = 1'bz;

endmodule

